/// ==============================================================================================
/// Namespace   : ScipBe.Controls.WinForms
/// Class       : Win32Helper & Win32
/// Version     : 1.1
/// Author      : Stefan Cruysberghs
/// Website     : www.scip.be
/// Date        : September 2006 - May 2007
/// Description : A FontManager stores lists of installed, theme and recently used fonts
/// Status      : Win32 API functions to retrieve extra font information
/// ==============================================================================================

using System;
using System.Drawing;
using System.Runtime.InteropServices;

namespace ScipBe.Controls.WinForms
{
  #region Win32Helper class
  /// <summary>
  /// 
  /// </summary>
  internal class Win32Helper : Object
  {
    private Graphics graphics;
    private IntPtr hdc;

    /// <summary>
    /// 
    /// </summary>
    public Win32Helper(Graphics graphics)
  	{
      this.graphics = graphics;
      hdc = graphics.GetHdc();
    }

    /// <summary>
    /// 
    /// </summary>
    ~Win32Helper()
  	{
      //graphics.ReleaseHdc(hdc);
    }

    /// <summary>
    /// 
    /// </summary>
    public PanoseFontFamilyTypes GetPanoseFontFamilyType(Font font)
    {
      byte bFamilyType = 0;

      IntPtr hFontOld = Win32.SelectObject(hdc, font.ToHfont());

      int bufSize = Win32.GetOutlineTextMetrics(hdc, 0, IntPtr.Zero);
      IntPtr lpOtm = Marshal.AllocCoTaskMem(bufSize);
      Marshal.WriteInt32(lpOtm, bufSize);
      int success = Win32.GetOutlineTextMetrics(hdc, bufSize, lpOtm);

      if (success != 0)
      {
        int offset = 61;
        bFamilyType = Marshal.ReadByte(lpOtm, offset);
      }

      Marshal.FreeCoTaskMem(lpOtm);

      Win32.SelectObject(hdc, hFontOld);

      return (PanoseFontFamilyTypes) bFamilyType;
    }
  }
  #endregion

  #region Win32 Panose Font Family Types
  /// <summary>
  /// Enumeration of Panose Font Family Types.  These can be used for
  /// determining the similarity of two fonts or for detecting non-character
  /// fonts like WingDings.
  /// </summary>
  /// <remarks>
  /// http://www.earlsoft.co.uk/api/
  /// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/gdi/fontext_48aa.asp
  /// <remarks>
  public enum PanoseFontFamilyTypes : int
  {
    /// <summary>
    /// Any
    /// </summary>
    PAN_ANY = 0,
    /// <summary>
    /// No Fit
    /// </summary>
    PAN_NO_FIT = 1,
    /// <summary>
    /// Text and Display
    /// </summary>
    PAN_FAMILY_TEXT_DISPLAY = 2,
    /// <summary>
    /// Script
    /// </summary>
    PAN_FAMILY_SCRIPT = 3,
    /// <summary>
    /// Decorative
    /// </summary>
    PAN_FAMILY_DECORATIVE = 4,
    /// <summary>
    /// Pictorial
    /// </summary>
    PAN_FAMILY_PICTORIAL = 5
  }
  #endregion

  #region Win32 class

  public partial class Win32
  {
    #region Win32 GDI32 functions
    /// <summary>
    /// The SelectObject function selects an object into the specified device context (DC)
    /// </summary>    
    /// <remarks>
    /// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/gdi/devcons_9v3o.asp
    /// </remarks>
    [DllImport("gdi32")]
    public static extern IntPtr SelectObject(
      IntPtr hdc,                 // handle to DC
      IntPtr hgdiobj              // handle to object   
    );
    /// <summary>
    /// The GetOutlineTextMetrics function retrieves text metrics for TrueType fonts
    /// </summary> 
    /// <remarks>
    /// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/gdi/fontext_6lyr.asp
    /// </remarks>
    [DllImport("gdi32", CharSet = CharSet.Ansi)]
    public static extern int GetOutlineTextMetrics(
      IntPtr hdc,				          // handle to DC
      int cbData,				          // size in bytes for text metrics
      IntPtr lpOtm			          // pointer to buffer to receive outline text metrics structure
    );
    #endregion
  }
  #endregion
}
